from datetime import date, timedelta

from oerppy import query, util
from oerppy.addons import project
from oerppy.addons.canonical import const, model
from oerppy.reg import registry


class CanonicalQuery(query.Query):

    def __init__(self, *args, **kwargs):
        super(CanonicalQuery, self).__init__(*args, **kwargs)
        self._cache = {}

    def views(self):
        return const.TIMESHEET_VIEWS

    def query(self, report, domains=None):
        """
        A dispatch method that runs the query for the given report name, and
        optionally, domains.

        Note that this is used primarily by the GUI, though since it's here,
        the command line export scripts use is as well.
        """
        if not domains:
            domains = []
        if report == const.TIMESHEET_HOURS:
            records = self.get_timesheet_hours(domains)
        elif report == const.TIMESHEETS_WEEKLY:
            records = self.get_all_weeks_oem_timesheets(domains)
        else:
            raise OpenERPPyParameterError("Unknown value for 'report.'")
        return records

    def get_users(self):
        raw_results = self.client.searchfields(
            entity='res.users',
            query=[],
            fields=[])
        return model.UsersSet(raw_results)

    def get_user_names(self, user_ids):
        """
        Get the OpenERP user names for the user IDs provided.
        """
        raw_results = self.client.searchfields(
            'res.users' ,
            [('id','in', user_ids)],
            ['id', 'name'])
        return model.UsersSet(raw_results)

    def get_user_dept(self):
        user_dept = self._cache.get("user_dept")
        if user_dept:
            return user_dept
        user_dept = registry.get_addon("hr").get_departments()
        self._cache["user_dept"] = user_dept
        return user_dept

    def get_entity_fields(self, entity_name):
        """
        Query an entity, returning just the column/field names.
        """
        raw_results = self.client.searchfields(
            entity=entity_name,
            query=[],
            fields=[])
        return raw_results[0].keys()

    def get_oem_users(self):
        """
        Get the IDs for everyone who uses the OEM timesheets.
        """
        raw_results = self.client.searchfields(
            entity='canonical.user.category',
            query=[],
            fields=[])
        results = model.UserCategoriesSet(raw_results)
        return results

    def get_oem_user_ids(self):
        """
        Get the IDs for everyone who uses the OEM timesheets.
        """
        user_ids = self._cache.get("oem_user_ids")
        if user_ids:
            return user_ids
        raw_results = self.client.searchfields(
            entity='canonical.user.category',
            query=[('code','=','OEMTIMESHEET')],
            fields=['user_ids'])
        result = model.UserCategoriesSet(raw_results).get_first()
        self._cache["oem_user_ids"] = result.user_ids
        return result.user_ids

    def get_timesheet_hours(self, domains=None):
        if not domains:
            domains = []
        raw_results = self.client.searchfields(
            entity="project.task.work",
            query=domains,
            fields=[])
        return model.TaskWorkSet(raw_results, self.get_user_dept())

    def get_timeentry_week_range(self, start_date, end_date):
        """
        Get timeentry data for a particular range.
        """
        raw_results = self.client.searchfields(
            entity="project.timeentry.week",
            query=[
                (['week','>',start_date]),
                ('week','<=', end_date)
                ],
            fields=[])
        return project.model.TimeEntryWeekSet(raw_results)

    def get_all_timeentry_weeks(self):
        """
        Get timeentry data since we've been tracking it.
        """
        start_date = "2010-12-31"
        today = date.today()
        last_week = today - timedelta(days=-7)
        return self.get_timeentry_week_range(
            start_date, last_week.isoformat())

    def get_users_timeentry(self, user_ids, date):
        """
        Get the timeentry for a given week.
        """
        raw_results = self.client.searchfields(
            'project.timeentry',
            [('week_start', '=', date),('user_id', 'in', user_ids)],
            [])
        return project.model.TimeEntrySet(raw_results)

    def _get_total_times(self, timesheets):
        """
        Create a lookup for total time by user ID for the provided timesheets.
        """
        total_time = {}
        if timesheets:
            for timesheet in timesheets:
                total_time[timesheet.user.key] = {
                    'time': timesheet.total_time,
                    'date': timesheet.week_start,
                    'state': timesheet.state}
        return total_time

    def _get_users_time_data(self, timesheets, user_ids, week_date=""):
        users = registry.get_addon("base").get_user_names(user_ids)
        # store lookup of total time by user for this week
        total_time = self._get_total_times(timesheets)
        for user_id in user_ids:
            if not week_date:
                week_date = total_time.get("date")
            record = {
                'Week Commencing': week_date,
                'User': users.get_user(user_id).name,
                'Department': self.get_user_dept().get(user_id, '?')}
            if total_time.get(user_id):
                record['Total Time'] = total_time.get(
                    user_id,{}).get('time',0)
                record['State'] = total_time[user_id]['state'].capitalize()
            else:
                # missing timesheet
                record['Total Time'] = 0
                record['State'] = 'Missing'
        return record
    
    def get_oem_timesheets(self, weeks, user_ids):
        """
        Get the timesheet data for the provided weeks and user IDs.
        """
        records = []
        for week in weeks:
            # get timesheets for the week
            timesheets = self.get_users_timeentry(user_ids, week.date)
            record = self._get_users_time_data(user_ids, week.date)
            records.append(record)
        return records

    def get_all_weeks_oem_timesheets(self):
        """
        Get the weekly timeentry records, highlighting the missing weeks.
        """
        # get user IDs of timesheet users and the weeks we're interested in
        weeks = self.get_all_timeentry_weeks()
        user_ids = self.get_oem_user_ids()
        return self.get_oem_timesheets(weeks, user_ids)

    def get_month_project_timesheets(self, month, year, project_name, 
                                     user_ids=None):
        mondays = util.get_mondays(date(year, month, 1), return_iso=True)
        if not user_ids:
            user_ids = registry.get_addon("project").get_project_user_ids(
                project_name=project_name)
        all_timesheets = []
        records = []
        for monday in mondays:
            week_timesheets = self.get_users_timeentry(user_ids, monday)
            for timesheet in week_timesheets:
                for project_data in timesheet.projects:
                    if project_data.project.value != project_name:
                        continue
                    # we want the following data points: user name, project
                    # name, date, and hours
                    row = {
                        "project_name": project_name,
                        "user": timesheet.user.value,
                        "date": timesheet.week_start,
                        "week_total_hours": timesheet.total_time,
                        "task_category": project_data.task.value,
                        "work_category": project_data.work.value,
                        "hours": project_data.hours}
                    records.append(row)
        return records

    def get_month_timesheets(self, month, year, user_ids=None):
        records = []
        project_names = registry.get_addon("project").get_project_names()
        for project_name in project_names:
            project_records = self.get_month_project_timesheets(
                month, year, project_name)
            records.extend(project_records)
        return records
