from datetime import datetime

from oerppy import config
from oerppy.scripting import ExportScript
from oerppy.addons.canonical import const


class MonthExport(ExportScript):
    """
    Generate an export of a given month's timesheet data.
    """
    def get_option_parser(self):
        """
        Options for monthly exports.
        """
        now = datetime.now()
        parser = super(MonthExport, self).get_option_parser()
        parser.add_option(
            "-m", "--month", dest="month", action="store", type="int",
            default=now.month,
            help="the month to export (default is current month)")
        parser.add_option(
            "-y", "--year", dest="year", action="store", type="int",
            default=now.year,
            help="the year to export (default is current year)")
        parser.add_option(
            "-p", "--project", dest="project_name", action="store",
            default="all",
            help="the project to get timesheet data for (if not provided, "
                 "all projects will be returned).")
        return parser

    def run(self):
        # we need to set purge to true here, since the options will have
        # already been gotten by the parent class at this time, and we need the
        # options of this class added to those
        options = self.get_options(purge=True)
        addon = self.service.get_addon("canonical")
        project_name = options.project_name
        if project_name == "all":
            data = addon.get_month_timesheets(options.month, options.year)
        else:
            data = addon.get_month_project_timesheets(
                options.month, options.year, project_name)
        self.export(data)


class AllWeeksExport(ExportScript):
    """
    Generate an export of each week's timesheet data since OpenERP was
    installed at Canonical.
    """
    def get_option_parser(self):
        """
        Options for all-weeks exports.
        """
        now = datetime.now()
        parser = super(AllWeeksExport, self).get_option_parser()
        parser.add_option(
            "-s", "--report-style", dest="report_style", action="store",
            default=const.TIMESHEETS_WEEKLY,
            help=("the report style to present in export (e.g., 'hours' " 
                  "or 'weekly'"))
        return parser

    def run(self):
        # we need to set purge to true here, since the options will have
        # already been gotten by the parent class at this time, and we need the
        # options of this class added to those
        options = self.get_options(purge=True)
        addon = self.service.get_addon("canonical")
        data = addon.query(options.report_style)
        self.export(data)


class OEMUsersExport(ExportScript):
    """
    Export a list of OEM Users.
    """
    def run(self):
        # we need to set purge to true here, since the options will have
        # already been gotten by the parent class at this time, and we need the
        # options of this class added to those
        options = self.get_options(purge=True)
        addon = self.service.get_addon("canonical")
        data = addon.get_oem_users()
        self.export(data)
