import base64, datetime, time, xmlrpclib

from oerppy import exceptions
from oerppy.util import parse_results

#   * client.Parser (parse the responses from the server and convert them to
#     model objects; optional)
#   * client.Model (object wrappers for results; optional)


class Client(object):
    """
    Connection details for OpenERP.
    """
    def __init__(self, endpoints, dbname, creds):
        self.endpoints = endpoints
        self.dbname = dbname
        self.credentials = creds
        self.connect()

    def connect(self):
        """
        Login to OpenERP, set the user ID, and prepare the proxies' callables.
        """
        uid = self.endpoints.common.login(self.dbname, self.credentials)
        if uid == False:
            raise exceptions.OpenERPPyLoginFailure()
        self.credentials.set_uid(uid)

        # cache the object proxy and prepare the execute callable
        self.endpoints.object.get_proxy()
        self.endpoints.object.prepare(self.dbname, self.credentials)

        # do the same for the report proxy
        self.endpoints.report.get_proxy()
        self.endpoints.report.prepare(self.dbname, self.credentials)
        return uid

    @parse_results
    def create(self, entity, record):
        """Create an entity"""
        return self.endpoints.object.execute(entity, "create", record)

    @parse_results
    def search(self, entity, query):
        return self.endpoints.object.execute(entity, "search", query)

    @parse_results
    def update(self, entity, record, ids):
        """Update an entity"""
        return self.endpoints.object.execute(entity, "write", ids, record)

    # preserve parity with publicly defined OpenERP API
    write = update

    @parse_results
    def delete(self, entity, ids):
        """Delete an entity"""
        return self.endpoints.object.execute(entity, "unlink", ids)

    # preserve parity with publicly defined OpenERP API
    unlink = delete

    @parse_results
    def read(self, entity, ids, fields):
        """
        Read an entity when provided a list of Ids
        """
        return self.endpoints.object.execute(entity, "read", ids, fields)

    @parse_results
    def searchfields(self, entity="", query="", fields=None):
        """
        """
        ids = self.search(entity, query)
        if ids:
            # Get the fields for the records
            records = self.endpoints.object.execute(entity, "read", ids,
                fields)
            return records
        else:
            return None

    @parse_results
    def report(self, model, ids, report_name, form):
        # Get the model's records and context
        if not ids:
            ids = self.endpoints.object.execute(model, "search",[])
        context = self.endpoints.object.execute("res.users", "context_get")
        # Run the report
        data = {
                "model": model,
                "form": form,
            }
        rep_id = self.endpoints.report.execute(report_name, ids, data, context)

        # Get the PDF
        state = False
        while not state:
            val = self.endpoints.report.get(rep_id)
            state = val["state"]
            if not state:
                time.sleep(1)

        #content = base64.decodestring(val["result"])
        #return content

        # Return full dictionary i.e. result=base-64 content, format=file
        # extension
        return val
