class Registry(object):
    """
    Registry instances will be most useful when made as singletons at the
    module level or on long-lived objects.
    """
    def __init__(self, a_dict=None):
        if not a_dict:
            a_dict = {}
        self._reg = a_dict

    def update(self, a_dict):
        self._reg.update(a_dict)

    def add(self, key, value):
        self._reg[key] = value

    def remove(self, key):
        del self._reg[key]

    def get_value(self, key):
        return self._reg[key]

    def get_key(self, value):
        for key, val in self._reg.items():
            if val == value:
                return key


class AddOnRegistry(Registry):
    """
    This is specifically for keeping track of addons. This will enable any
    script writer to import and incorporate addons from anywhere in the Python
    path.
    """
    def __init__(self, *args, **kwargs):
        super(AddOnRegistry, self).__init__(*args, **kwargs)
        self._order = []
        self.are_instantiated = False

    def add(self, name, instance):
        super(AddOnRegistry, self).add(name, instance)
        self._order.append(name)

    def get_addon(self, name):
        return self.get_value(name)

    def remove_addon(self, name):
        self.remove(name)

    def get_addon_names(self):
        return self._order

    def get_addons(self):
        addons = []
        for name in self._order:
            addons.append(self.get_value(name))
        return addons

    def instantiate(self, client):
        """
        This method updates the registry, replacing the addon classes with
        actual instances.

        Though intended for use by the service instance, anything that has
        access to the client instance can use it.
        """
        class_pairs = zip(self._order, self.get_addons())
        instance_pairs = [(name, klass(client)) for name, klass in class_pairs]
        self.update(dict(instance_pairs))
        self.are_instantiated = True


# Default registry is for addons; we can change this later, if a general
# registry is needed, and then make the addon registry a sub-reg of the general
# one.
registry = AddOnRegistry()
