from optparse import OptionParser
import sys

from oerppy import config, const, export, service


class Script(object):
    """
    The common code for all scripting classes.
    """
    def __init__(self, config_data):
        self.options = None
        self.service = None
        self.set_service(config_data)

    def set_service(self, config_data):
        options = self.get_options()
        server_config = config.get_server(options.host, options.dbname)
        url = server_config.get("url")
        user = server_config.get("username")
        password = server_config.get("password")
        self.service = service.Service(url, options.dbname, user, password)
        # note that we could add the addon (self) to the registry at this
        # point, but we'd have to hard code a name here, as opposed to how it's
        # currently done (the executable setting the name associated with the
        # addon when it's added to the registry).

    def get_option_parser(self):
        """
        Options common to all scripts.
        """
        parser = OptionParser(usage=self.get_usage())
        parser.add_option(
            "-H", "--host", dest="host", action="store",
            help="the host to perform the config lookup for")
        parser.add_option(
            "-D", "--database", dest="dbname", action="store",
            help="the db name to perform the config lookup for")
        return parser

    def get_usage(self):
        usage = "Usage: %prog [options]"
        description = self.__doc__
        return "%s\n%s" % (usage, description)

    def get_options(self, purge=False):
        if self.options and purge == False:
            return self.options
        parser = self.get_option_parser()
        (options, args) = parser.parse_args()
        if None in [options.host, options.dbname]:
            parser.error("Options -H and -D are both required. "
                         "For more information, use --help.")
        self.options = options
        return self.options


class ExportScript(Script):
    """
    This class serves to provide a place for adding export-specific methods and
    data.
    """
    def get_option_parser(self):
        """
        Options common to all export scripts.
        """
        parser = super(ExportScript, self).get_option_parser()
        parser.add_option(
            "-o", "--output-file", dest="output_filename", action="store",
            default=const.STDOUT_FILENAME,
            help="the file to write the export to (default is stdout)")
        parser.add_option(
            "-f", "--format", dest="export_format", action="store",
            default=const.EXPORT_RAW,
            help="the format to export the data in (default is JSON)")
        return parser

    def export(self, data, *args, **kwargs):
        # we need to set purge to true here, since the options will have
        # already been gotten by the parent class at this time, and we need the
        # options of this class added to those
        options = self.get_options(purge=True)
        # create the filehandle
        if options.output_filename == const.STDOUT_FILENAME:
            fd = sys.stdout
        else:
            fd = open(options.output_filename, "w+")
        # dispatch the appropriate export based on the format option
        if options.export_format == const.EXPORT_RAW:
            import json
            fd.write(json.dumps(data))
        elif options.export_format == const.EXPORT_CSV:
            export.export_csv(data, fd, *args, **kwargs)
        #elif options.export_format == const.EXPORT_GOOGLE_DOCS:
        #    export.export_google(user, pass, data, ..., ?)


class OpenERPUsersExport(ExportScript):
    """
    Export a list of OpenERP users.
    """
    def run(self):
        addon = self.service.get_addon("base")
        data = addon.get_users()
        self.export(data)


class OpenERPProjectsExport(ExportScript):
    """
    Export a list of OpenERP projects.
    """
    def run(self):
        addon = self.service.get_addon("project")
        data = addon.get_project_names()
        self.export(data)
