import xmlrpclib

from oerppy import client, exceptions, reg, util


class Credentials(object):
    """
    """
    def __init__(self, user, password):
        self.user = user
        self.password = password
        self.uid = None

    def set_uid(self, uid):
        self.uid = uid


class Endpoint(object):
    """
    """
    def __init__(self, uri=""):
        self._uri = uri
        self._proxy = None
        self.host = ""
        self.port = None
        self.path = "/" 
        self._parse_uri(uri)
        if not self.scheme:
            self.scheme = "http"

    def _parse_uri(self, uri):
        scheme, host, port, path = util.parse(
            str(uri), defaultPort=False)
        self.scheme = scheme
        self.host = host
        self.port = port
        self.path = path

    def set_host(self, host):
        self.host = host

    def get_host(self):
        return self.host

    def get_canonical_host(self):
        """ 
        Return the canonical host as for the Host HTTP header specification.
        """
        host = self.host.lower()
        if self.port is not None:
            host = "%s:%s" % (host, self.port)
        return host

    def set_canonical_host(self, canonical_host):
        """ 
        Set host and port from a canonical host string as for the Host HTTP
        header specification.
        """
        parts = canonical_host.lower().split(":")
        self.host = parts[0]
        if len(parts) > 1 and parts[1]:
            self.port = int(parts[1])
        else:
            self.port = None

    def set_path(self, path):
        self.path = path

    def get_uri(self):
        """
        Get a URL representation of the service.
        """
        return "%s://%s%s" % (
            self.scheme, self.get_canonical_host(), self.path)

    def get_proxy(self, purge=False):
        """
        Get an XML-RPC proxy for the endpoint.
        """
        uri = self.get_uri()
        if purge or type(self._proxy) == type(None):
            self._proxy = xmlrpclib.ServerProxy(uri, allow_none=1)
        return self._proxy


class CommonEndpoint(Endpoint):
    """
    """
    def __init__(self, *args, **kwargs):
        super(CommonEndpoint, self).__init__(*args, **kwargs)
        self._parse_uri(self._uri + "/xmlrpc/common")
        if not self.scheme:
            self.scheme = "http"

    def login(self, dbname, creds):
        """
        Returns the UID of the user upon successful login.
        """
        proxy = self.get_proxy()
        return proxy.login(dbname, creds.user, creds.password)


class ObjectEndpoint(Endpoint):
    """
    """
    def __init__(self, *args, **kwargs):
        super(ObjectEndpoint, self).__init__(*args, **kwargs)
        self._parse_uri(self._uri + "/xmlrpc/object")
        if not self.scheme:
            self.scheme = "http"
        # the 'execute' attribute will be set to a partial of the proxy's
        # execute method
        self.execute = None

    def prepare(self, dbname, creds, *args, **kwargs):
        proxy = self.get_proxy()
        self.execute = util.partial(
            proxy.execute, dbname, creds.uid, creds.password)


class ReportEndpoint(Endpoint):
    """
    """
    def __init__(self, *args, **kwargs):
        super(ReportEndpoint, self).__init__(*args, **kwargs)
        self._parse_uri(self._uri + "/xmlrpc/report")
        if not self.scheme:
            self.scheme = "http"
        # the 'execute' and 'get' attributes will be set to partials of the
        # proxy's report and report_get methods, respectively
        self.execute = None
        self.get = None

    def prepare(self, dbname, creds, *args, **kwargs):
        proxy = self.get_proxy()
        self.execute = util.partial(
            proxy.report, dbname, creds.uid, creds.password)
        self.get = util.partial(
            proxy.report_get, dbname, creds.uid, creds.password)


class ServiceEndpoints(object):

    def __init__(self, url):
        self.common = CommonEndpoint(url)
        self.object = ObjectEndpoint(url)
        self.report = ReportEndpoint(url)


class Service(object):
    """
    """
    def __init__(self, url, dbname, user, password):
        for variable in [url, dbname, user, password]:
            if not variable:
                raise exceptions.OpenERPPyConfigError(
                    "url, dbname, user and password must "
                    "all have values set.")
        self.endpoints = ServiceEndpoints(url)
        self.dbname = dbname
        self.credentials = Credentials(user, password)
        self.client = client.Client(
            self.endpoints, self.dbname, self.credentials)
        # now that the client is defined, let's start up the addons
        reg.registry.instantiate(self.client)

    def __getattr__(self, name):
        """
        This bit of "magic" allows us to make client and addon method calls
        directly on instances of this service class. The lookup order is the
        following:
            * check the Service instance for the attribute, then
            * check the Client instance (self.client), then
            * check each addon that hass been added to the Client instance

        Here are some examples:
            * self.dbname returns the value of dbname on the Service instance

            * self.addons isn't found on the Service instance (self) so it
              check the client; self.client.addons is returned

            * self.query isn't found on self or self.client, so each addon is
              checked (in the order that they were added to the registry, so
              add your most important ones first); depending upon which addons
              you have loaded, the first one with a "query" method will be
              returned.

        If you have overlap between the addon methods (i.e., addons with
        methods in common that you want to be able to call), then you should
        call the method directly on the addon. See the docstring for
        Service.get_addon for more information.
        """
        try:
            attrib = super(Service, self).__getattr__(name)
        except AttributeError:
            try:
                attrib = getattr(self.client, name)
            except AttributeError:
                for addon in reg.registry.get_addons():
                    try:
                        attrib = getattr(addon, name)
                    except AttributeError:
                        pass
        if not attrib:
            raise AttributeError(
                "Attribute '%s' not found on Service, Client, "
                "or any Addon." % (name))

    def get_addon(self, name):
        """
        This method is a convenience wrapper around the registry method of the
        same name. If you want to be very explicit about making calls on a
        particular addon, rather than rely on the magic of __getattr__, then
        this method is for you. Simply get the addon that you want, and call
        methods on the returned object like usual.
        """
        if not reg.registry.are_instantiated:
            return
        return reg.registry.get_addon(name)
