import datetime, os, tempfile, unittest
from urlparse import urlparse

from oerppy import const, util


class ConfigUtilTestCase(unittest.TestCase):

    def setUp(self):
        self.src = os.path.abspath("../" + const.SAMPLE_CONFIG)
        self.fd, self.fullpath = tempfile.mkstemp()

    def test_read_config(self):
        data = util.read_config(self.src, self.fullpath)
        self.assertEqual(data, open(self.src).read())


class ParseUrlTestCase(unittest.TestCase):
    """
    Test URL parsing facility and defaults values.
    """
    def test_parse(self):
        """
        L{parse} correctly parses a URL into its various components.
        """
        # The default port for HTTP is 80.
        self.assertEqual(
            util.parse("http://127.0.0.1/"),
            ("http", "127.0.0.1", 80, "/"))

        # The default port for HTTPS is 443.
        self.assertEqual(
            util.parse("https://127.0.0.1/"),
            ("https", "127.0.0.1", 443, "/"))

        # Specifying a port.
        self.assertEqual(
            util.parse("http://spam:12345/"),
            ("http", "spam", 12345, "/"))

        # Weird (but commonly accepted) structure uses default port.
        self.assertEqual(
            util.parse("http://spam:/"),
            ("http", "spam", 80, "/"))

        # Spaces in the hostname are trimmed, the default path is /.
        self.assertEqual(
            util.parse("http://foo "),
            ("http", "foo", 80, "/"))

    def test_external_unicode_interference(self):
        """
        L{parse} should return C{str} for the scheme, host, and path
        elements of its return tuple, even when passed an URL which has
        previously been passed to L{urlparse} as a C{unicode} string.
        """
        badInput = u"http://example1.com/path"
        goodInput = badInput.encode("ascii")
        urlparse(badInput)


class PartialTestCase(unittest.TestCase):

    def test_partial(self):
        def myfunction(known1, known2, unknown1, unknown2):
            return (known1, known2, unknown1, unknown2)

        curried = util.partial(myfunction, "this", "that")
        result = curried("dunno", "no clue")
        self.assertEqual(result, ('this', 'that', 'dunno', 'no clue'))


class DateTestCase(unittest.TestCase):

    def test_get_datetime(self):
       date = "2011-01-01"
       result = util.get_datetime(date)
       self.assertEqual(result, datetime.date(2011, 1, 1))

    def test_start_of_week_with_iso(self):
       date = "2011-01-01"
       result = util.start_of_week(date)
       self.assertEqual(result, datetime.date(2010, 12, 27))

    def test_start_of_week_with_iso_returning_iso(self):
       date = "2011-01-01"
       result = util.start_of_week(date, return_iso=True)
       self.assertEqual(result, "2010-12-27")

    def test_start_of_week_with_datetime(self):
       date = datetime.date(2011, 1, 1)
       result = util.start_of_week(date)
       self.assertEqual(result, datetime.date(2010, 12, 27))

    def test_start_of_week_with_datetime_returning_iso(self):
       date = datetime.date(2011, 1, 1)
       result = util.start_of_week(date, return_iso=True)
       self.assertEqual(result, "2010-12-27")

    def test_get_mondays(self):
        date = datetime.date(2011,1,1)
        result = util.get_mondays(date)
        self.assertEqual(
            result, 
            [datetime.date(2011, 1, 3),
             datetime.date(2011, 1, 10), 
             datetime.date(2011, 1, 17), 
             datetime.date(2011, 1, 24), 
             datetime.date(2011, 1, 31)])
        result = util.get_mondays("2011-02-01", return_iso=True)
        self.assertEqual(
            result, 
            ['2011-02-07', '2011-02-14', '2011-02-21', '2011-02-28'])
        result = util.get_mondays("2011-02-28", return_iso=True)
        self.assertEqual(
            result,
            ['2011-02-07', '2011-02-14', '2011-02-21', '2011-02-28'])
        result = util.get_mondays("2011-08-01", return_iso=True)
        self.assertEqual(
            result, 
            ['2011-08-01', '2011-08-08', '2011-08-15', '2011-08-22',
             '2011-08-29'])
        result = util.get_mondays("2011-08-31", return_iso=True)
        self.assertEqual(
            result, 
            ['2011-08-01', '2011-08-08', '2011-08-15', '2011-08-22',
             '2011-08-29'])
        result = util.get_mondays("2011-09-21", return_iso=True)
        self.assertEqual(
            result,
            ['2011-09-05', '2011-09-12', '2011-09-19', '2011-09-26'])
