import os
import calendar
import datetime
from urlparse import urlparse, urlunparse
import xmlrpclib

from oerppy import const, exceptions


MONDAY = 0


def write_default_config(dest_dir=""):
    """
    Create a default config file.

    This function's parameters are used only for testing purposes.
    """
    src_file = os.path.abspath(const.CONFIG_TEMPLATE)
    if not os.path.isfile(src_file):
        raise exceptions.OpenERPPyConfigError(
            "Could not locate default configuration file '%s'." % src_file)
    src = open(src_file)
    data = src.read()
    src.close()
    basedir = os.path.expanduser(dest_dir or const.CONFIG_DIR)
    if not os.path.exists(basedir):
        os.makedirs(basedir)
    dest_file = os.path.join(basedir, const.CONFIG_FILE)
    dest = open(dest_file, "w+")
    dest.write(data)
    dest.close()
    # XXX set the perms on it to 600


def read_config(default_config="", dest_dir=""):
    """
    Read the config file; if it's not there, create a default one and then
    re-attempt a read.

    This function's parameters are used only for testing purposes.
    """
    config_file = os.path.expanduser(default_config or const.DEFAULT_CONFIG)
    if not os.path.isfile(config_file):
        write_default_config(dest_dir)
    config = open(config_file)
    data = config.read()
    config.close()
    return data


def parse(url, defaultPort=True):
    """
    Split the given URL into the scheme, host, port, and path.

    @type url: C{str}
    @param url: An URL to parse.

    @type defaultPort: C{bool}
    @param defaultPort: Whether to return the default port associated with the
        scheme in the given url, when the url doesn't specify one.

    @return: A four-tuple of the scheme, host, port, and path of the URL.  All
    of these are C{str} instances except for port, which is an C{int}.
    """
    url = url.strip()
    parsed = urlparse(url)
    scheme = parsed[0]
    path = urlunparse(("", "") + parsed[2:])
    host = parsed[1]

    if ":" in host:
        host, port = host.split(":")
        try:
            port = int(port)
        except ValueError:
            # A non-numeric port was given, it will be replaced with
            # an appropriate default value if defaultPort is True
            port = None
    else:
            port = None

    if port is None and defaultPort:
        if scheme == "https":
            port = 443
        else:
            port = 80

    if path == "":
        path = "/"
    return (str(scheme), str(host), port, str(path))


def partial(func, *args, **kwargs):
    """
    By using partial function application, we're able "pre-process" functions
    whose first few arguments we know ahead of time.

    As used in this project, partial saves us extra lines of code in the form
    of redundancy removal. We need only "prepare" the functions/methods ahead
    of time, and then "finish" them at actual execution time, instead of
    re-calling the original method over and over again with half of the
    arguments remaining the same every time.
    """
    def newfunc(*fargs, **fkeywords):
        newkeywords = kwargs.copy()
        newkeywords.update(fkeywords)
        return func(*(args + fargs), **newkeywords)
    newfunc.func = func
    newfunc.args = args
    newfunc.keywords = kwargs
    return newfunc


def parse_results(func):
    """
    This is a decorator for methods that perform remote calls against the
    OpenERP XML-RPC server. It provides the ability to intercept results and
    have them parsed.
    """
    def checker(*args, **kwargs):
        try:
            results = func(*args, **kwargs)
            return results
        except xmlrpclib.Fault, error:
            raise exceptions.OpenERPPyRemoteError(error.faultCode)
    return checker


def get_datetime(date):
    # check to see if the date is in ISO date format
    if isinstance(date, basestring):
        date_object = datetime.date(
            int(date[:4]), int(date[5:7]), int(date[8:10]))
    # or if it's a date object already
    elif isinstance(date, datetime.date):
        date_object = date
    return date_object


def start_of_week(date, return_iso=False):
    date_object = get_datetime(date)
    monday = date_object + datetime.timedelta(days=-date_object.weekday())
    if return_iso:
        monday = monday.isoformat()
    return monday


def get_mondays(date, return_iso=False):
    date_object = get_datetime(date)
    _get_date = partial(datetime.date, date_object.year, date_object.month)
    month = calendar.monthcalendar(date_object.year, date_object.month)
    mondays = []
    for week in month:
        # days outside of the month are given the value 0; we skip those
        day = week[MONDAY]
        if day == 0:
            continue
        monday = _get_date(day)
        if return_iso:
            monday = monday.isoformat()
        mondays.append(monday)
    return mondays
